/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * Instance.java
 *
 * Created on 20 gennaio 2003, 12.50
 * @author  Valter Crescenzi
 */

package roadrunner;

import java.util.*;

import roadrunner.util.Indenter;
import roadrunner.ast.ASTAnd;
import roadrunner.ast.ASTVariant;
import roadrunner.parser.Token;

public class Instance  {
  
  private ASTAnd root; // root of instance tree
  //N.B. *token of instance* to node of expression mapping; inclusive of null tokens and exclusive of invariant token
  private IdentityHashMap token2node; 
  private Wrapper wrappedby;
  private Sample sample;
  
  /** Creates a new instance of class Instance */
  public Instance(Wrapper wrapper, Sample sample, ASTAnd root, IdentityHashMap token2node) {
    this.wrappedby = wrapper;
    this.sample = sample;
    this.root = root;
    this.token2node = token2node;
  }
  
  public ASTAnd getRoot() {
    return this.root;
  }
  
  public Wrapper getWrapper() {
    return this.wrappedby;
  }
  
  public Sample getSample() {
    return this.sample;
  }

  public IdentityHashMap getTokens2Nodes() {
      return this.token2node;
  }
  
  public String getLabel(Token token) {
    ASTVariant node = (ASTVariant)this.token2node.get(token);
    return node.getLabel();
  }
  
  public String dump(String prefix) {
    return this.getRoot().dump(new Indenter(prefix));
  }
  
  public String toString() {
    return dump("");
  }
  
}
