/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * WrappersRepository.java
 *
 *
 * Created on 17 dicembre 2002, 11.55
 * @author  Valter Crescenzi
 */

package roadrunner;

import java.util.*;

import roadrunner.config.Config;
import roadrunner.ast.Expression;
import roadrunner.engine.Registry;
import roadrunner.engine.MDLEvaluator;
import roadrunner.parser.BindingException;

public class WrappersRepository {
    
    private Map wrapper2samples;
    private Map wrapper2instances;
    private Registry registry;
    
    /** Creates a new instance of WrappersRepository */
    //Passare a WrappersRepository(Set wrappers, Sample[] samples)
    // C'e' qualcosa che non va tra questa classe, Engine che la usa e Registry che viene usata
    public WrappersRepository(Registry registry, Sample[] samples) {
        this.wrapper2samples = new LinkedHashMap();
        this.wrapper2instances = new LinkedHashMap();
        this.registry = registry;
        // make Wrappers
        Set wrappers = new LinkedHashSet();
        Iterator it = registry.getExpressionsByCompressRatio().iterator();
        while (it.hasNext()) {
            Expression exp = (Expression)it.next();
            wrappers.add(new Wrapper(exp, Config.getInstance().getLexicalPrefs()));
        }
        putAll(wrappers,Arrays.asList(samples));
    }
    
    public boolean isEmpty() {
        return this.wrapper2samples.isEmpty();
    }
    
    public Set getWrappers() {
        return Collections.unmodifiableSet(this.wrapper2samples.keySet());
    }
    
    public Set getSamples(Wrapper wrapper) {
        return Collections.unmodifiableSet((Set)this.wrapper2samples.get(wrapper));
    }
    
    public Set getInstances(Wrapper wrapper) throws BindingException {
        if (this.wrapper2instances.containsKey(wrapper))
            return (Set)this.wrapper2instances.get(wrapper);
        Set result = new LinkedHashSet();
        Set samples = getSamples(wrapper);
        Iterator it = samples.iterator();
        while (it.hasNext()) {
            Sample sample = (Sample)it.next();
            Instance instance = wrapper.wrap(sample);
            result.add(instance);
        }
        this.wrapper2instances.put(wrapper, result);
        return result;
    }
    
    private void putAll(Set wrappers, Collection samples) {
        Iterator it = wrappers.iterator();
        while (it.hasNext()) this.putAll((Wrapper)it.next(), samples);
    }

    private void putAll(Wrapper wrapper, Collection samples) {
        this.wrapper2samples.put(wrapper, new LinkedHashSet(samples));
    }
    
    public MDLEvaluator getMDLEvaluator(Wrapper wrapper) {
        return this.registry.getMDLEvaluator(wrapper.getExpression());
    }
    
    public void setWrappersNamesIds(String name) {
        Iterator it = getWrappers().iterator();
        int n=0;
        while (it.hasNext()) {
            Wrapper wrapper = (Wrapper)it.next();
            wrapper.setName(name);
            wrapper.setId(n++);
        }
    }
}
