/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * ASTBuilder.java
 *
 * Created on 31 gennaio 2003, 11.22
 * @author  Valter Crescenzi
 */

package roadrunner.ast;

import java.util.*;

import roadrunner.parser.Token;
import roadrunner.parser.token.TokenFactory;

public class ASTBuilder {
    
    // This class should be rewritten by scratch
    
    /** Creates a new instance of ASTBuilder */
    private Stack stack;
    private ASTAnd root;
    
    private TokenFactory factory;
    
    public ASTBuilder() {
        this.stack = new Stack();
        this.factory = TokenFactory.getInstance();
        this.root = null;
    }
    
    public ASTAnd getRoot() {
        return this.root;
    }
    
    public void startNodeScope(Node node) {
        stack.push(node.jjtGetChildren());
    }
    public void endNodeScope(Node node) {
        Object popped = this.stack.pop();
        if (popped!=node.jjtGetChildren())
            throw new RuntimeException("Popped element is not the expected one!");
    }
    
    public void addNode(Node node) {
        if (root!=null) {
            List cl = (List)stack.peek();
            cl.add(node);
        }
        else root = (ASTAnd)node;
    }
    
    public void startNodeScope() {
        List templist = new LinkedList();
        stack.push(templist);
    }
    
    public void discardScope() {
        stack.pop();
    }
    
    public void terminateScope(Node node) {
        List templist = (List)stack.pop();
        SimpleNode sn = (SimpleNode)node;
        sn.jjtGetChildren().addAll(templist);
        if (stack.isEmpty()) this.root = (ASTAnd)node;
    }
    
    public Node createTokenNode(Token token) {
        return (Node)token;
    }
    
    public Node createNullToken(Token sample) {
        return (Node)factory.createNullToken(sample);
    }
    
    public ASTVariant createVariant(Token token, String name) {
        ASTVariant result = new ASTVariant(token);
        result.setLabel(name);
        return result;
    }
    public ASTVariant createVariant(Token token) {
        ASTVariant result = new ASTVariant(token);
        return result;
    }
    public ASTSubtree createSubtree(Token token) {
        return new ASTSubtree(token);
    }
    public ASTAnd createAnd() {
        return new ASTAnd();
    }
    public ASTPlus createPlus() {
        return new ASTPlus();
    }
    public ASTHook createHook() {
        return new ASTHook();
    }
    public Node createNode(Node node) {
        Node clone = node.clone(false);
        if (clone instanceof SimpleNode) {// what abot a method getComposite()?
            SimpleNode sn = (SimpleNode)clone;
            sn.jjtGetChildren().clear();
        }
        return clone;
    }
    
}
