/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/**
 * ASTSubtree.java
 * Created: 
 *
 * @author  Valter Crescenzi
 * @version
 */
package roadrunner.ast;

import roadrunner.parser.Token;
import roadrunner.parser.TokenListRegion;

public class ASTSubtree extends NodeAdapter implements Matchable {
    
    private Token token;
    
    ASTSubtree(Token token) {
        super(ASTConstants.JJTSUBTREE);
        this.token = token;
    }
    
    /** Accept the visitor. **/
    public boolean jjtAccept(Visitor visitor) {
        return visitor.visit(this);
    }
    
    public Token getRootToken() {
        return this.token;
    }
        
    /* this should align with subtree starting from the given depth */
    public boolean matches(Token t) {
        return t.depth()>this.getRootToken().depth();
    }
    
    public int depth() {
        return getRootToken().depth();
    }
    
    public int code() {
        return getRootToken().code();
    }
    /** SUPPORTO AL WRAPPING **/
    /* Segue supporto del wrapping che VA tolto da qui in qualche modo */
    private TokenListRegion matching;
    
    public void setMatchingTokenList(TokenListRegion matching) {
        this.matching = matching;
    }
    
    public TokenListRegion getMatchingTokenList() {
        return this.matching;
    }
    /** FINE SUPPORTO AL WRAPPING **/
    
    public boolean equals(Object o) {
        if (!super.equals(o)) return false;
        ASTSubtree that = (ASTSubtree)o;
        return this.getRootToken().equals(that.getRootToken());
    }
    
    public int hashCode() {
        return this.getTypeId()+getRootToken().hashCode();
    }
    
    public String toString() {
        return "  _/"+getRootToken().depth()+"\\_";
    }
}
