/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * VisitorPlugger.java
 *
 * Created on 16 ottobre 2003, 9.34
 * @author  Valter Crescenzi
 */

package roadrunner.ast;


import java.util.*;

import roadrunner.bidi.Region;
import roadrunner.bidi.*;

public class VisitorPlugger implements Visitor {
    
    /** Creates a new instance of VisitorPlugger */
    
    protected Direction dir;
    
    protected VisitorPlugger() {
        this(Direction.LEFT2RIGHT);
    }
    protected VisitorPlugger(Direction dir) {
        this.dir = dir;
    }
    
    public boolean visit(Expression exp) {
        return visit(exp.getRoot());
    }
    
    public boolean visit(Region region) {
        return visitList(region.asList());
    }
    
    public boolean visit(NodeAdapter node) {
        throw new RuntimeException("This method shouldn't be called on "+node.getClass());
    }
    
    public boolean visit(ASTToken node)   {  return visitNode(node);       }
    public boolean visit(ASTVariant node) {  return visitNode(node);       }
    public boolean visit(ASTSubtree node) {  return visitNode(node);       }
    public boolean visit(ASTHook node)    {  return visitNode(node);       }
    public boolean visit(ASTPlus node)    {  return visitNode(node);       }
    public boolean visit(ASTAnd node)     {  return visitNode(node);       }
    
    protected boolean visitNode(Node node){   return visitChildren(node);  }
    
    protected boolean visitChildren(Node node) {
        return visitList(node.jjtGetChildren());
    }
    
    protected boolean visitList(List list) {
        // N.B. this is the only method which is invoked on all nodes; visitNode() and visitChildren are not invoked on node of base region
        boolean result = true;
        ListIterator it = BidirectionalListFactory.newListIterator(list, this.dir);
        startVisitingList(it);
        while (it.hasNext()) {
            Node node = (Node)it.next();
            result = node.jjtAccept(this) && result; // AND logic for result
        }
        endVisitingList(it);
        return result;
    }
    protected void startVisitingList(ListIterator it) {};
    protected void   endVisitingList(ListIterator it) {};
    
}
