/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/**
 * Direction.java
 * an index refers to an element: valid indices for a list of N elements are 0,1...N-1;
 * a position refers to a "hole" between elements: valid positions are 0,1...N;
 * therefore element of index i is located between position i and i+1 
 *
 * Created: Sun Nov 21 18:58:01 1999
 *
 * @author Valter Crescenzi
 * @version
 */

package roadrunner.bidi;

import java.util.List;
import java.util.ListIterator;

public interface BidirectionalList extends List {
    
  public ListIterator listIterator(Direction dir);
  public ListIterator listIterator(Direction dir, int pos);
  
  public List subListTo(int position);  
  public List subListToIndex(int index); // element of index "index" excluded
  public List subListFrom(int position);
    
  /* to parametrize with respect to the direction */ 
  public Object getFirst();
  public Object getLast();

}

