/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * Registry.java
 * This class is used to order expression by Compress-Ratio
 * Created 
 * @author  Valter Crescenzi
 */
package roadrunner.engine;

import java.util.*;

import roadrunner.ast.Expression;
import roadrunner.parser.TokenList;

public class Registry {
    
    private Set expressions;
    private TokenList[] tokenlists;
    
    private Map expression2evaluator;
        
    /** Creates a new instance of Registry*/
    Registry(Set expressions, TokenList[] tokenlists) {
        this.expression2evaluator = new HashMap();
        this.expressions = expressions;
        this.tokenlists = tokenlists;
    }
    
    public Set getExpressions()        { return this.expressions; }
    
    public MDLEvaluator getMDLEvaluator(Expression expression) {
        MDLEvaluator evaluator = (MDLEvaluator)expression2evaluator.get(expression);
        if (evaluator==null) evaluator = new MDLEvaluator(expression, this.tokenlists);
        expression2evaluator.put(expression, evaluator);
        return evaluator;
    }
    
    private double getCompressRatio(Expression expression) {
        return getMDLEvaluator(expression).getCompressRatio();
    }

    public SortedSet getExpressionsByCompressRatio() {
        // sort by DL
        SortedSet result = new TreeSet(
        new Comparator() {
            
            public int compare(Object o1, Object o2) {
                Expression exp1 = (Expression)o1, exp2 = (Expression)o2;
                double cr1 = getCompressRatio(exp1), cr2 = getCompressRatio(exp2);
                return (cr1>cr2? +1 : (cr1==cr2 ? exp1.hashCode()-exp2.hashCode() : -1));
            }
            
            public boolean equals(Object obj) { return false; }
        });
        result.addAll(this.getExpressions());
        return result;
    }
    
}
