/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * SquareOperator.java
 *
 * Created on 23 marzo 2003, 15.09
 * @author  Valter Crescenzi
 */

package roadrunner.engine.space.operator;

import java.util.logging.*;

import roadrunner.bidi.Region;
import roadrunner.bidi.Direction;
import roadrunner.parser.MismatchPoint;
import roadrunner.parser.Parser;
import roadrunner.parser.TokenList;
import roadrunner.ast.Expression;
import roadrunner.ast.ExpressionRegion;
import roadrunner.util.Indenter;
import roadrunner.engine.space.Operator;
import roadrunner.engine.sampler.CharacteristicSampler;


abstract class SquareOperator extends SimpleOperator implements Operator {
    
    static private Logger log = Logger.getLogger(SquareOperator.class.getName());
    
    private Searcher searcher;
    Region square;
    int k;
    
    int lenOfMatchingBorder;
    
    /** Creates a new instance of SimpleOperator */
    SquareOperator(MismatchPoint mismatch, Searcher searcher, Region square, int k) {
        super(mismatch);
        this.expired = square.isEmpty();
        this.searcher = searcher;
        this.square = square;
        this.k = k;
    }
    
    Searcher getSearcher() {
        return this.searcher;
    }
    
    Region getRegionBeforeSquare() { return mismatch.getMatchingExpression(); }
    abstract Expression getCandidateSquare();
    abstract Region getRegionAfterSquare();
    Region getSquare()             { return this.square;                      }
    abstract ExpressionRegion getExtension(); 
        
    int countMatchingBorder(Direction dir, Region r1, Region r2, int maxLength) {
        if (maxLength==0) return 0;
        int result = countMatchingTokens(dir, r1, r2, maxLength) + countMatchingTokens(dir, r2, r1, maxLength);
        return result;
    }
    
    private int countMatchingTokens(Direction dir, Region r, Region forSampling, int maxLength) {
        CharacteristicSampler sampler = new CharacteristicSampler(forSampling, dir);
        sampler.setMaxSampleLength(maxLength);
        return countMatchingTokens(dir, r, (TokenList)sampler.computeNext());
    }
    
    private int countMatchingTokens(Direction dir, Region r, TokenList tl) {
        Parser parser = new Parser(r, dir);
        return  parser.parsePrefix(tl);
    }
    
    // give priority to smaller squares: this is a problem for nested border pluses (they need a square composed of all sub-squares).
    // otherwise the simpler first-occurrence operators would be pushed after the corresponding k-occurrence ones (k>1)
    public int h() { return 3*mismatch.getOrder()-this.lenOfMatchingBorder+k; }
    
    public String toString(Indenter ind) {
        return ind+"Operator "+getId()+":\t"+getClass().getName()+"^"+k+"\th="+h()+"\n"+
               ind+"Mismatch "+mismatch.getId()+":\t"+mismatch.toString()+"\n"+
               ind+"  Square:\t"+getSquare()+"\n";
    }
    
}
