/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * addSubtree.java
 *
 * Created on 14 aprile 2003, 16.09
 * @author  Valter Crescenzi
 */

package roadrunner.engine.space.operator;

import java.util.*;

import roadrunner.bidi.*;
import roadrunner.ast.*;
import roadrunner.parser.*;
import roadrunner.engine.space.Operator;
import roadrunner.util.Indenter;

class addSubtree extends SimpleOperator implements Operator {
    
    static class Factory extends OperatorFactory {
        
        private Factory(MismatchPoint m) {
            super(m);
        }
        
        public List createOperators() {
            return Collections.singletonList(new addSubtree(this.m));
        }
    }
    
    static OperatorFactory getFactory(MismatchPoint m) {
        return new Factory(m);
    }
    
    private ExpressionRegion extension;
    
    private addSubtree(MismatchPoint mismatch) {
        super(mismatch);
        this.extension = null;
    }
    
    protected Expression compute() {
        this.expired = true; // just one result
        this.extension = getExtension();
        if (extension==null) return null; // no way
        try {
            return mismatch.getExpression().addSubtree(extension);
        }
        catch (RuntimeException rte) {
            System.err.println(this);
            System.err.println(rte.getMessage());
            System.exit(-1);
            return null;
        }
    }
    
    protected ExpressionRegion getExtension() {
        List baselist = mismatch.getBaseRegion().asList();
        int pos = mismatch.getPosBeforeFirstMismatchingNode();
        int depth = mismatch.getUpperDOMdepth();
        BidirectionalList bidilist = BidirectionalListFactory.newListView(Direction.LEFT2RIGHT, baselist);
        ListIterator leftIt  = bidilist.listIterator(Direction.RIGHT2LEFT, pos);
        ListIterator rightIt = bidilist.listIterator(Direction.LEFT2RIGHT, pos);
        int left   = getPosBeforeStartTagAtGivenDepth(leftIt, depth);
        if (left==-1) return null;
        int right  = getPosAfterEndTagAtGivenDepth(rightIt, depth);
        if (right==-1) return null;
        this.extension = (ExpressionRegion)mismatch.getBaseRegion().subRegion(left, right);
        if (!delimitSubtree(extension))
            throw new RuntimeException("This region does not delimit a subtree!\n"+extension);
        return extension;
    }
    
    private boolean delimitSubtree(ExpressionRegion extension) {
        // check if extension covers a whole subtree
        Token f = (Token)extension.getFirst();
        Token l = (Token)extension.getLast();
        return (f.code()+l.code()==0 && f.isStartTag() && l.isEndTag() && f.depth()==l.depth());
    }
    
    private int getPosBeforeStartTagAtGivenDepth(ListIterator lit, int depthWanted) {
        while (lit.hasNext()) {
            Node node = (Node)lit.next();
            if (!(node instanceof Token)) continue;
            Token token = (Token) node;
            int depthReached = token.depth();
            if (depthReached==depthWanted && token.isStartTag())
                return Direction.LEFT2RIGHT.posBeforeIndex(lit.previousIndex());
        }
        return -1;
    }

    private int getPosAfterEndTagAtGivenDepth(ListIterator lit, int depthWanted) {
        while (lit.hasNext()) {
            Node node = (Node)lit.next();
            if (!(node instanceof Token)) continue;
            Token token = (Token) node;
            int depthReached = token.depth();
            if (depthReached==depthWanted && token.isEndTag())
                return Direction.LEFT2RIGHT.posAfterIndex(lit.previousIndex());
        }
        return -1;
    }
    
    public int h() { return 1000000000-mismatch.getUpperDOMdepth(); }
    
    public String toString(Indenter ind) {
        return ind+"Operator "+getId()+":\t"+getClass().getName()+"\th="+h()+"\n"+
               ind+"Mismatch "+mismatch.getId()+":\t"+mismatch.toString()+"\n"+
               ind+"Extension:\t"+getExtension()+"\n";
    }
    
}
