/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * Binder.java
 *
 * Created on 23 gennaio 2003, 12.13
 * @author  Valter Crescenzi
 */

package roadrunner.parser;

import java.util.*;

import roadrunner.bidi.*;
import roadrunner.ast.*;

public class Binder extends ParserListenerAdapter implements ParserListener {
    
    private Parser parser;
    private Stack stack;
    private IdentityHashMap tokenBinds;
    private IdentityHashMap subtreeBinds;
    private TokenList tokenlist;
    
    /** Creates a new instance of Binder */
    public Binder(Expression exp) {
        this.parser = new Parser(exp);
        this.parser.setParserListener(this);
        this.tokenBinds = null;
        this.subtreeBinds = null;
        this.stack = null;
    }
    
    public void setBindings(TokenList tl) throws BindingException {
        this.tokenlist = tl;
        this.stack = new Stack();
        this.tokenBinds = new IdentityHashMap(); //Class Token overrides method equals
        this.subtreeBinds = new IdentityHashMap(); //TokenList must not override equals, preserve order
        if (!parser.parse(tl))
            throw new BindingException(parser.getMismatches());
    }
    
    public IdentityHashMap getTokenBindings() throws BindingException {
        return tokenBinds;
    }
    
    public IdentityHashMap getSubtreeBindings() throws BindingException {
        return subtreeBinds;
    }
    
    public void startAnd(ASTAnd node) {
        this.stack.push(this.tokenBinds);
        this.stack.push(this.subtreeBinds);
        this.tokenBinds = new IdentityHashMap();
        this.subtreeBinds = new IdentityHashMap();
    }
    
    public void endAnd(ASTAnd node, boolean matches) {
        Map squareTokenBinds = this.tokenBinds;
        Map squareSubtreeBinds = this.subtreeBinds;
        this.subtreeBinds = (IdentityHashMap)this.stack.pop();
        this.tokenBinds = (IdentityHashMap)this.stack.pop();
        if (matches) {
            this.tokenBinds.putAll(squareTokenBinds);
            this.subtreeBinds.putAll(squareSubtreeBinds);
        }
    }
    
    public void endVariant(ASTVariant node, boolean matches, Token token) {
        if (matches) this.tokenBinds.put(token,node);
    }
    
    public void endToken(ASTToken node, boolean matches, Token token) {
        if (matches) this.tokenBinds.put(token,node);
    }
    
    
    public void endSubtree(ASTSubtree node, boolean matches, int startIndex, int endIndex) {
        /* extract TokenListRegion */
        if (matches) this.subtreeBinds.put(extractTokenList(startIndex-1,endIndex+1), node);
    }
    
    private TokenList extractTokenList(int startIndex, int endIndex) {
        Direction d = Direction.LEFT2RIGHT;
        TokenListRegion tlr = this.tokenlist.asRegion();
        TokenList result = new TokenList(d.subRegion(tlr, d.posBeforeIndex(startIndex), d.posAfterIndex(endIndex)).asList());
        return result;
    }
    
}
