/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * ParserListener.java
 * This interface defines the set of events generated by Parser
 *
 * @author  Valter Crescenzi
 * Created on 24 aprile 2003, 18.00
 */

package roadrunner.parser;

import roadrunner.ast.*;

public interface ParserListener {
    
  public void startAnd(ASTAnd and);
  public void endAnd(ASTAnd and, boolean matches);
  
  public void startPlus(ASTPlus plus);
  public void endPlus(ASTPlus plus, boolean matches, int times);
  
  public void startHook(ASTHook hook);
  public void endHook(ASTHook hook, boolean matches, int times);
  
  public void startSubtree(ASTSubtree subtree);
  public void endSubtree(ASTSubtree subtree, boolean matches, int firstIndex, int lastIndex);
  
  public void startVariant(ASTVariant variant);
  public void endVariant(ASTVariant variant, boolean matches, Token token);
  
  public void startToken(ASTToken node);
  public void endToken(ASTToken node, boolean matches, Token token);
  
}
