/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * Tag.java
 *
 * Created on December 25, 2003, 12:39 PM
 * @author  Valter Crescenzi
 */

package roadrunner.parser.token;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import roadrunner.config.Preferences;

class Tag {
    
    private String element;
    private Map attributes;
    private int code;
    private int hashcode;
    
    /** Creates a new instance of Tag */
    Tag(String element, Map attributes) {
        this.element = element;
        this.attributes = attributes;
        this.code = 0;
        this.hashcode = computeHashCode();
    }
    
    public String getElement() {
        return this.element;
    }
    public Map getAttributes() {
        return this.attributes;
    }
    
    int code() {
        if (this.code==0) this.code = TagFactory.getInstance().getTagCode(this);
        return this.code;
    }
    
    public int hashCode() {
        return this.hashcode;
    }
    
    private int computeHashCode() {
        return this.element.hashCode()+this.attributes.keySet().hashCode();
    }
    
    public boolean equals(Object o) {
        if (this.hashCode()!=o.hashCode()) return false;
        Tag t = (Tag) o;
        return this.element.equals(t.element) && this.attributes.equals(t.attributes);
    }
    
    public String toString() {
        StringBuffer result = new StringBuffer(getElement());
        Iterator attrsIt = getAttributes().entrySet().iterator();
        if (attrsIt.hasNext()) printAttribute(result,(Map.Entry)attrsIt.next());
        while (attrsIt.hasNext()) {
            printAttribute(result,(Map.Entry)attrsIt.next());
        }
        return result.toString();
    }
    
    private void printAttribute(StringBuffer result, Map.Entry entry) {
        result.append(" ");
        String name  = (String)entry.getKey();
        String value = (String)entry.getValue();
        result.append(name + (value!=null ? ":"+value : ""));
    }
}
