/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * Util.java
 *
 * Created on 
 * @author  Valter Crescenzi
 */

package roadrunner.util;

import java.io.*;
import java.util.*;

public class Util {
    
    
    public static File searchInRRHOME(String filename) throws FileNotFoundException {
        String paths = System.getProperty("rr.home",".");
        StringTokenizer tokenizer = new StringTokenizer(paths,File.pathSeparator);
        File file = null;
        while (tokenizer.hasMoreTokens()) {
            String path = tokenizer.nextToken();
            file = new File(path,filename);
            if (file.canRead()) return file.getAbsoluteFile();
        }
        throw new FileNotFoundException("Required file "+ filename +"cannot be read from "+ paths);
    }
    
    static public void copyFile(File from , File to) throws IOException {
        Reader reader = new BufferedReader(new FileReader(from));
        Writer writer = new BufferedWriter(new FileWriter(to));
        int ch;
        while ((ch=reader.read())!=-1) {
            writer.write(ch);
        }
        reader.close(); writer.close();
    }
    
    static public void reverseMapping(Map value2keys, Map map) {
        Iterator it = map.entrySet().iterator();
        while (it.hasNext()) {
            Map.Entry entry = (Map.Entry)it.next();
            Object value = entry.getValue();
            Object key = entry.getKey();
            List keys = (List)value2keys.get(value);
            if (keys==null) keys = new LinkedList();
            keys.add(key);
            value2keys.put(value, keys);
        }
    }
    
    static public List getKeys(IdentityHashMap map, Object value) {
        //Cannot use:
        //Map clone = (Map)map.clone();
        //clone.values().retainAll(Collections.singleton(value));
        //return clone.keySet();
        // because identity on view is based on equals method
        List result = new LinkedList();
        Iterator it = map.entrySet().iterator();
        while (it.hasNext()) {
            Map.Entry entry = (Map.Entry)it.next();
            if (entry.getValue()==value)
                result.add(entry.getKey());
        }
        return result;
    }
    
    public static boolean isAllWhiteSpaceChars(String string) {
        final char nbsp = (char)160;
        int len = string.length();
        for (int h=0; h<len; h++) {
            char ch = string.charAt(h);
            if (!(Character.isWhitespace(ch) || ch==nbsp)) return false;
        }
        return true;
    }
    //GRAZIE A NEKOHTML QUESTA DOVREBBE ESSERE DIVENTATA UNICA
    static public String escapeChars4XML(String s) {
        int len = s.length();
        StringBuffer result = null;
        for(int i=0; i<len; i++) {
            char ch = s.charAt(i);
            if (result==null) {
                switch(ch) {
                    case '&':
                    case '=':
                    case '<':
                    case '>':
                    case '\u00a0':
                        result = new StringBuffer(s.substring(0,i));
                }
            }
            if (result!=null) {
                switch(ch) {
                    case '&':
                        result.append("&amp;");
                        break;
                    case '=':
                        result.append("%3d");
                        break;
                    case '<':
                        result.append("&lt;");
                        break;
                    case '>':
                        result.append("&gt;");
                        break;
                    case '\u00a0':
                        result.append(" ");
                        break;
                    default:
                        result.append(ch);
                }
            }
        }
        return (result==null ? s : result.toString());
    }
    
}



