/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * ASTBuilderTest.java
 * JUnit based test
 *
 * Created on 27 febbraio 2003, 11.12
 * @author Valter Crescenzi
 */

package roadrunner.ast;

import junit.framework.*;

import java.util.*;

import roadrunner.parser.*;
import roadrunner.parser.token.*;

public class ASTBuilderTest extends TestCase {
    
    public ASTBuilderTest(java.lang.String testName) {
        super(testName);
    }
    
    public static void main(java.lang.String[] args) {
        junit.textui.TestRunner.run(suite());
    }
    
    public static Test suite() {
        TestSuite suite = new TestSuite(ASTBuilderTest.class);
        
        return suite;
    }
    
    /** Test of class roadrunner.ast.ASTBuilder. */
    public void testBuilder() {
        System.out.println("testBuilder");
        System.out.println(getFixExpression().getRoot().dump());
    }
    
    public static Expression getFixExpression() {
        ASTBuilder builder = new ASTBuilder();
        TokenFactory factory = TokenFactory.getInstance();
        
        //      And
        //   /    |    \         \
        //  A    Hook   C  Variant
        //        \
        //        And
        //         \
        //          B
        
        ASTAnd and = builder.createAnd();
        builder.addNode(and);
        builder.startNodeScope(and);
        Node token1 = builder.createTokenNode(factory.createTextToken("A",0));
        builder.addNode(token1);
        ASTHook hook = builder.createHook();
        builder.addNode(hook);
        builder.startNodeScope(hook);
        ASTAnd and2 = builder.createAnd();
        builder.addNode(and2);
        builder.startNodeScope(and2);
        Node token2 = builder.createTokenNode(factory.createTextToken("B",0));
        builder.addNode(token2);
        builder.endNodeScope(and2);
        builder.endNodeScope(hook);
        Node token3 = builder.createTokenNode(factory.createTextToken("C",0));
        ASTVariant variant = builder.createVariant(factory.createOpenTagToken("IMG",Collections.EMPTY_MAP,0), "label");
        builder.addNode(token3);
        builder.addNode(variant);
        builder.endNodeScope(and);
        Node root = builder.getRoot();
        assertEquals(and,root);
        assertEquals(token1,root.jjtGetChild(0));
        assertEquals(token2,root.jjtGetChild(1).jjtGetChild(0).jjtGetChild(0));
        assertEquals(token3,root.jjtGetChild(2));
        return new Expression((ASTAnd)root);
    }
    
    public static Expression getFixExpressionWithPlus() {
        ASTBuilder builder = new ASTBuilder();
        TokenFactory factory = TokenFactory.getInstance();
        //      And
        //   /    |    \         \
        //  A    Plus  C  Variant
        //        \
        //        And
        //         \
        //          B
        
        ASTAnd and = builder.createAnd();
        builder.addNode(and);
        builder.startNodeScope(and);
        Node token1 = builder.createTokenNode(factory.createTextToken("A",0));
        builder.addNode(token1);
        ASTPlus plus = builder.createPlus();
        builder.addNode(plus);
        builder.startNodeScope(plus);
        ASTAnd and2 = builder.createAnd();
        builder.addNode(and2);
        builder.startNodeScope(and2);
        Node token2 = builder.createTokenNode(factory.createTextToken("B",0));
        builder.addNode(token2);
        builder.endNodeScope(and2);
        builder.endNodeScope(plus);
        Node token3 = builder.createTokenNode(factory.createTextToken("C",0));
        ASTVariant variant = builder.createVariant(factory.createOpenTagToken("IMG",Collections.EMPTY_MAP,0), "label");
        builder.addNode(token3);
        builder.addNode(variant);
        builder.endNodeScope(and);
        Node root = builder.getRoot();
        assertEquals(and,root);
        assertEquals(token1,root.jjtGetChild(0));
        assertEquals(token2,root.jjtGetChild(1).jjtGetChild(0).jjtGetChild(0));
        assertEquals(token3,root.jjtGetChild(2));
        return new Expression((ASTAnd)root);
    }
    
}
