/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * ExpressionTest.java
 * JUnit based test
 *
 * Created on 16 marzo 2003, 10.55
 * @author Valter Crescenzi
 */

package roadrunner.ast;

import junit.framework.*;

import java.util.*;
import java.util.logging.*;

import roadrunner.Sample;
import roadrunner.bidi.*;
import roadrunner.parser.*;
import roadrunner.parser.token.*;

public class ExpressionTest extends TestCase {
      
  private final static Logger log = Logger.getLogger(ExpressionTest.class.getName());
      
  public ExpressionTest(java.lang.String testName) {
    super(testName);
  }
  
  public static void main(java.lang.String[] args) {
    junit.textui.TestRunner.run(suite());
  }
  
  public static Test suite() {
    TestSuite suite = new TestSuite(ExpressionTest.class);
    
    return suite;
  }
  
  private Expression exp;
  private Node.Path pathOfB;
  private Region regionOfB;
  private Region regionOfCVar;
  private TokenFactory factory;
  
  protected void setUp() {
    this.exp = ASTBuilderTest.getFixExpression();
    int [] indicesOfBpath = {1,0,0};
    this.pathOfB = new PathArray(indicesOfBpath);
    int [] indicesOfBparent = {1,0};
    Node.Path pathOfBparent = new PathArray(indicesOfBparent);
    this.regionOfB = new ExpressionRegion(this.exp, pathOfBparent, 0, 1);
    this.regionOfCVar = new ExpressionRegion(this.exp, Node.EMPTY_PATH, 2, 4);
    this.factory = TokenFactory.getInstance();
  }
  
  //         And
  //   /    |    \   \
  //  A    Hook   C   Variant
  //        \
  //        And
  //         \
  //          B
  
  
  /** Test of asRegion method, of class roadrunner.ast.Expression. */
  public void testAsRegion() {
    System.out.println("testAsRegion");
    List list = this.exp.asRegion().asList(this.exp);
    Iterator it = list.iterator();
    int one[] = new int[1];
    one[0]=0;
    //
    assertEquals(this.exp.getNode(new PathArray(one)), it.next());
    one[0]++;
    //
    assertEquals(this.exp.getNode(new PathArray(one)), it.next());
    one[0]++;
    //
    assertEquals(this.exp.getNode(new PathArray(one)), it.next());
    one[0]++;
    //
    assertEquals(this.exp.getNode(new PathArray(one)), it.next());
    one[0]++;    
  }
  
  /** Test of replaceNode method, of class roadrunner.ast.Expression. */
  public void testReplaceNode() {
    System.out.println("testReplaceNode");
    Node newNode = factory.createTextToken("new text", 2);
    this.exp.replaceNode(this.pathOfB, newNode);
    assertEquals(exp.getNode(pathOfB), newNode);
  }
  
  /** Test of addPlus method, of class roadrunner.ast.Expression. */
  public void testAddPlus() {
    System.out.println("testAddPlus");
    
    Node underThePlus = factory.createTextToken("under the plus", 0);
    ASTAnd newSquare = new ASTAnd(Collections.singletonList(underThePlus));
    
    Expression result = this.exp.addPlus((ExpressionRegion)regionOfB, new Expression(newSquare));
    int indices[] = {1,0,0,0,0};
    Node.Path underThePlusPath = new PathArray(indices);
    log.info(result.dump("addPlus>"));
    assertEquals(result.getNode(underThePlusPath),underThePlus);
  }
  
  /** Test of addHook method, of class roadrunner.ast.Expression. */
  public void testAddHook() {
    System.out.println("testAddHook");
    
    Node underTheHook = factory.createTextToken("under the hook", 0);
    ASTAnd newSquare = new ASTAnd(Collections.singletonList(underTheHook));
    
    Expression result = this.exp.addHook((ExpressionRegion)regionOfB, new Expression(newSquare));
    int indices[] = {1,0,0,0,0};
    Node.Path underTheHookPath = new PathArray(indices);
    log.info(result.dump("addHook>"));
    assertEquals(result.getNode(underTheHookPath),underTheHook);
  }
  
  public void testAddHook2() {
    System.out.println("testAddHook2");
    
    Node underTheHook2 = factory.createTextToken("under the hook2", 0);
    ASTAnd newSquare = new ASTAnd(Collections.singletonList(underTheHook2));
    
    Expression result = this.exp.addHook((ExpressionRegion)regionOfCVar, new Expression(newSquare));
    int indices[] = {2,0,0};
    Node.Path underTheHook2Path = new PathArray(indices);
    log.info(result.dump("addHook2>"));
    assertEquals(result.getNode(underTheHook2Path),underTheHook2);
  }
    
  public void testExpressionEquals() {
      System.out.println("testExpressionEquals");
      
      assertEquals(ASTBuilderTest.getFixExpression(), 
                   ASTBuilderTest.getFixExpression());
      assertEquals(ASTBuilderTest.getFixExpressionWithPlus(), 
                   ASTBuilderTest.getFixExpressionWithPlus());
      assertFalse(ASTBuilderTest.getFixExpression().equals(ASTBuilderTest.getFixExpressionWithPlus()));
  }
  
  // Add test methods here, they have to start with 'test' name.
  // for example:
  // public void testHello() {}
  
}
