/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * BidirectionalListFactoryTest.java
 * JUnit based test
 *
 * Created on 28 febbraio 2003, 12.05
 * @author Valter Crescenzi
 */

package roadrunner.bidi;

import junit.framework.*;

import java.util.*;

public class BidirectionalListFactoryTest extends TestCase {
    
  private BidirectionalList l2r_bidilist1;
  private BidirectionalList r2l_bidilist2;
  private List list1;
  private List list2;
  private Object[] array1 = {"a","b","c"};
  private Object[] array2 = {new Integer(1), new Integer(2), new Integer(3) };
  
  public void setUp() {
    this.list1 = new LinkedList(Arrays.asList(array1));
    this.list2 = new LinkedList(Arrays.asList(array2));
    this.l2r_bidilist1 = BidirectionalListFactory.newListView(Direction.LEFT2RIGHT, list1);
    this.r2l_bidilist2 = BidirectionalListFactory.newListView(Direction.RIGHT2LEFT, list2);
  }
  
  public BidirectionalListFactoryTest(java.lang.String testName) {
    super(testName);
  }
  
  public static void main(java.lang.String[] args) {
    junit.textui.TestRunner.run(suite());
  }
  
  public static Test suite() {
    TestSuite suite = new TestSuite(BidirectionalListFactoryTest.class);
    
    return suite;
  }
  
  //  /** Test of X method, of class roadrunner.BidirectionalList */
  //  public void testX() {
  //    System.out.println("X");
  //
  //  }
  
  /** Test of listIterator method, of class roadrunner.BidirectionalList */
  public void testListIterator() {
    System.out.println("testListIterator");
    
    ListIterator lit1 = l2r_bidilist1.listIterator(Direction.RIGHT2LEFT);
    assertTrue(lit1.hasNext());
    assertEquals(lit1.next(),array1[2]);
    assertEquals(lit1.next(),array1[1]);
    assertEquals(lit1.next(),array1[0]);
    assertTrue(!lit1.hasNext());
    
    ListIterator lit3 = l2r_bidilist1.listIterator(Direction.LEFT2RIGHT);
    assertTrue(lit3.hasNext());
    assertEquals(lit3.next(),array1[0]);
    assertEquals(lit3.next(),array1[1]);
    assertEquals(lit3.next(),array1[2]);
    assertTrue(!lit3.hasNext());
    
    
    ListIterator lit2 = r2l_bidilist2.listIterator();
    assertTrue(lit2.hasNext());
    assertEquals(lit2.next(),array2[2]);
    assertEquals(lit2.next(),array2[1]);
    assertEquals(lit2.next(),array2[0]);
    assertTrue(!lit2.hasNext());

    ListIterator lit4 = r2l_bidilist2.listIterator(Direction.RIGHT2LEFT);
    assertTrue(lit4.hasNext());
    assertEquals(lit4.next(),array2[2]);
    assertEquals(lit4.next(),array2[1]);
    assertEquals(lit4.next(),array2[0]);
    assertTrue(!lit4.hasNext());
  
    ListIterator lit5 = r2l_bidilist2.listIterator(Direction.LEFT2RIGHT);
    assertTrue(lit5.hasNext());
    assertEquals(lit5.next(),array2[0]);
    assertEquals(lit5.next(),array2[1]);
    assertEquals(lit5.next(),array2[2]);
    assertTrue(!lit5.hasNext());
  }
  
  /** Test of subListTo methods, of class roadrunner.BidirectionalList */
  public void testSubListTo() {
    System.out.println("SubListTo");
    
    List sublist1 = l2r_bidilist1.subListTo(2);
    assertEquals(sublist1.size(),2);
    assertEquals(sublist1.get(0),array1[0]);
    assertEquals(sublist1.get(1),array1[1]);
    
    // Other direction...
    List sublist2 = r2l_bidilist2.subListTo(2);
    assertEquals(sublist2.size(),1);
    assertEquals(sublist2.get(0),array2[2]);
    
  }
  
  /** Test of subListToIndex methods, of class roadrunner.BidirectionalList */
  public void testSubListToIndex() {
    System.out.println("SubListToIndex");
    
    List sublist = l2r_bidilist1.subListToIndex(1);
    assertEquals(sublist.size(),1);
    assertEquals(sublist.get(0),array1[0]);

    // Other direction...
    List sublist2 = r2l_bidilist2.subListToIndex(2);
    assertEquals(sublist2.size(),0);    
  }
  
  
  /** Test of subListFrom methods, of class roadrunner.BidirectionalList */
  public void testSubListFrom() {
    System.out.println("SubListFrom");
    
    List sublist = l2r_bidilist1.subListFrom(2);
    assertEquals(sublist.size(),1);
    assertEquals(sublist.get(0),array1[2]);
    
    // Other direction...
    List sublist2 = r2l_bidilist2.subListFrom(1);
    assertEquals(sublist2.size(),1);
    assertEquals(sublist2.get(0),array2[0]);
  }
   
  
}
