/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * RealSearchSpaceTest.java
 * JUnit based test
 *
 * Created on 16 marzo 2003, 9.36
 */

package roadrunner.engine.space;

import junit.framework.*;

import java.net.URL;
import java.util.logging.*;

import roadrunner.ast.Expression;
import roadrunner.parser.Parser;
import roadrunner.parser.TokenList;
import roadrunner.Sample;
import roadrunner.config.*;

import roadrunner.FixtureOnFile;

/**
 *
 * @author Valter Crescenzi
 */
public class RealSearchSpaceTest extends SearchSpaceTest implements Constants {
    
    private final static Logger log = Logger.getLogger(RealSearchSpaceTest.class.getName());
        
    private Sample byname2 = createRealSample("rufus/2ByName.html");
    private Sample byname3 = createRealSample("rufus/3ByName.html");
    private Sample byname9 = createRealSample("rufus/9ByName.html");
    private Sample bynameY = createRealSample("rufus/YByName.html");
    
    public RealSearchSpaceTest(java.lang.String testName) throws Exception {
        super(testName);
    }
    
    private Sample createRealSample(String filename) throws Exception {
        return new Sample(new URL("file:"+FixtureOnFile.getSampleFile(filename)), Config.getPrefs());
    }
    
    public static void main(java.lang.String[] args) throws Exception {
        junit.textui.TestRunner.run(suite());
    }
    
    public static Test suite() {
        TestSuite suite = new TestSuite(RealSearchSpaceTest.class);
        return suite;
    }
    
    public void testWithRealSamples23() {
        System.out.println("*********************");
        System.out.println("testWithRealSamples23");
        System.out.println("*********************");
        
        testWithRealSamples(byname2, byname3);
    }
    
    public void testWithRealSamples29() {
        System.out.println("*********************");
        System.out.println("testWithRealSamples29");
        System.out.println("*********************");
        
        testWithRealSamples(byname2, byname9);
    }
    
    public void testWithRealSamples2Y() {
        System.out.println("*********************");
        System.out.println("testWithRealSamples2Y");
        System.out.println("*********************");
        
        testWithRealSamples(byname2, bynameY);
    }
    
    private void testWithRealSamples(Sample sample1, Sample sample2) {
        log.info("Real Samples: "+sample1+" "+sample2);
        Config.getPrefs().put(FREE_TEXT,"false");
        Config.getPrefs().put(FREE_PLUS,"false");
        Config.getPrefs().put(MAX_NUM_OCCUR,"1");
        Config.getPrefs().put(AMBIGUITY,"1");
        Expression result;
        Expression exp = new Expression(sample1.getTokenlist());
        TokenList  tl  = sample2.getTokenlist();
        
        SearchSpace space = new SearchSpace(exp, tl, 0);
        int count = 0;
        while (space.hasNext() && count<5) {
            result = space.next();
            log.info(result.dump("Rufus Solution "+count+") "));
            assertTrue(new Parser(result).parseAllExpression(tl));
            count++;
        }
    }
    
    // Add test methods here, they have to start with 'test' name.
    // for example:
    // public void testHello() {}
    
}
