/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * BoxTest.java
 * JUnit based test
 *
 * Created on December 14, 2003, 3:44 PM
 * @author Valter Crescenzi
 */

package roadrunner.labeller;

import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;
import java.util.*;
import junit.framework.*;

public class BoxTest extends TestCase {
    
    public BoxTest(java.lang.String testName) {
        super(testName);
    }
    
    private Box obox;
    
    public static Test suite() {
        TestSuite suite = new TestSuite(BoxTest.class);
        return suite;
    }
    
    public void setUp() {
        this.obox = new Box(0,0, 10, 10);
    }
    
    /** Test of getCenter method, of class roadrunner.labeller.Box. */
    public void testGetCenter() {
        System.out.println("testGetCenter");
        
        assertEquals(new Point2D.Double(5,5),this.obox.getCenter());
    }
    
    /** Test of distance method, of class roadrunner.labeller.Box. */
    public void testDistance() {
        System.out.println("testDistance");
        
        assertEquals("zero distance", 0.0, this.obox.distance(new Box(0,10, 10,20)), 0.0);
        assertEquals("horizontal distance", 80.0, this.obox.distance(new Box(90,0, 90,10)), 0.0);
        assertEquals("diagonal distance", 50.0, this.obox.distance(new Box(40,50, 50,80)), 0.0);
    }
    
    /** Test of alignment method, of class roadrunner.labeller.Box. */
    public void testAlignment() {
        System.out.println("testAlignment");
        
        assertEquals("perfectly horizontally aligned", 0, new Box(0,0,10,10).alignment(new Box(100,0,110,10)), 0.0);
        assertEquals("perfectly vertically aligned", 0, new Box(0,0,10,10).alignment(new Box(0,100,10,110)), 0.0);
        assertEquals("perfectly diagonally aligned", Math.PI/4, new Box(0,0,0,0).alignment(new Box(10,10,10,10)), 0.0001);
        assertEquals("centers are horizontally aligned", 0, new Box(100,100,130,130).alignment(new Box(200,110,300,120)), 0.0000001);
    }
    
    /** Test of isInBetween method, of class roadrunner.labeller.Box. */
    public void testIsInBetween() {
        System.out.println("testIsInBetween");
        
        //two boxes which do not overlap vertically
        Box box1 = new Box(0,10,10,20);
        Box box2 = new Box(20,30,30,40);
        assertTrue(new Box(10,20,20,30).isInBetween(box1,box2));
        assertTrue(new Box(10,20,20,30).isInBetween(box2,box1));
        assertFalse(new Box(15,10,20,20).isInBetween(box1,box2));
        assertFalse(new Box(15,10,20,20).isInBetween(box2,box1));
        
        //two boxes which overlap vertically
        Box box3 = new Box(0,0,10,40);
        Box box4 = new Box(50,10,60,30);
        assertTrue(new Box(20,15,40,25).isInBetween(box3,box4));
        assertTrue(new Box(20,15,40,25).isInBetween(box4,box3));
        assertFalse(new Box(20,0,40,8).isInBetween(box3,box4));
        assertFalse(new Box(20,0,40,8).isInBetween(box4,box3));
    }
    
    /** Test of compareTo method, of class roadrunner.labeller.Box. */
    public void testCompareTo() {
        System.out.println("testCompareTo");
        
        assertTrue(new Box(0,0,10,10).compareTo(new Box(30,30, 40, 40))<0);
        assertTrue(new Box(30,30, 40, 40).compareTo(new Box(0,0,10,10))>0);
        assertTrue(new Box(0,0, 10, 10).compareTo(new Box(0,0,10,10))==0);
        
        assertTrue(new Box(0,0, 10, 10).compareTo(new Box(0,1,10,10))<0);
        assertTrue(new Box(0,0, 10, 10).compareTo(new Box(1,0,10,10))<0);        
        assertTrue(new Box(0,0, 10, 10).compareTo(new Box(0,0,10,11))<0);        
        assertTrue(new Box(0,0, 10, 10).compareTo(new Box(0,0,11,10))<0);
        
        assertTrue(new Box(0,0, 0,0).compareTo(new Box(0,0,0,0))==0);       
    }
    
    // Add test methods here, they have to start with 'test' name.
    // for example:
    // public void testHello() {}
    public static void main(String[] args) {
        new BoxTest("test x debug").testAlignment();
    }
    
}
